@@ -4,7 +4,7 @@
  * GChemPaint library
  * document.cc 
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,26 +28,32 @@
 #include "document.h"
 #include "settings.h"
 #include "docprop.h"
+#include "fragment.h"
 #include "reaction.h"
 #include "mesomery.h"
+#include "molecule.h"
+#include "residue.h"
 #include "text.h"
 #include "theme.h"
 #include "tool.h"
 #include "window.h"
-#include <goffice/utils/go-rangefunc.h>
+#include <gcu/objprops.h>
+#include <gcu/loader.h>
+#include <goffice/math/go-rangefunc.h>
 #include <unistd.h>
 #include <libgen.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs-file-info.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
+#include <gio/gio.h>
 #include <glib/gi18n-lib.h>
 #include <openbabel/mol.h>
 #include <openbabel/obconversion.h>
 #include <clocale>
 #include <stack>
+#include <cstring>
 
 using namespace OpenBabel;
 using namespace std;
+using namespace gcu;
 
 namespace gcp {
 
@@ -149,7 +155,7 @@ GtkWidget* Document::GetWidget ()
 }
 
 
-const gchar* Document::GetTitle ()
+const gchar* Document::GetTitle () const
 {
 	if (m_title)
 		return m_title;
@@ -157,7 +163,7 @@ const gchar* Document::GetTitle ()
 		return GetLabel ();
 }
 
-const gchar* Document::GetLabel ()
+const gchar* Document::GetLabel () const
 {
 	return m_label;
 }
@@ -192,10 +198,10 @@ void Document::SetFileName (string const &Name, const gchar* mime_type)
 		m_label = g_strdup (m_filename + i);
 }
 
-void Document::BuildBondList (list<Bond*>& BondList, Object* obj)
+void Document::BuildBondList (list<Bond*>& BondList, Object const *obj) const
 {
-	Object* pObject;
-	map<string, Object*>::iterator i;
+	Object const *pObject;
+	map<string, Object*>::const_iterator i;
 	for (pObject = obj->GetFirstChild (i); pObject; pObject = obj->GetNextChild (i))
 		if (pObject->GetType () == gcu::BondType)
 			BondList.push_back ((Bond*)(*i).second);
@@ -267,15 +273,16 @@ bool Document::ImportOB (OBMol& Mol)
 			AddBond (pBond);
 		}
 	}
+	m_Empty = !HasChildren ();
 	if (m_Window)
 		m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
 	return true;
 }
 
-void Document::BuildAtomTable (map<string, unsigned>& AtomTable, Object* obj, unsigned& index)
+void Document::BuildAtomTable (map<string, unsigned>& AtomTable, Object const *obj, unsigned& index) const
 {
-	Object* pObject;
-	map<string, Object*>::iterator i;
+	Object const *pObject;
+	map<string, Object*>::const_iterator i;
 	for (pObject = obj->GetFirstChild (i); pObject; pObject = obj->GetNextChild (i))
 		if (pObject->GetType() == AtomType)
 			AtomTable[(*i).second->GetId ()] = index++;
@@ -283,7 +290,7 @@ void Document::BuildAtomTable (map<string, unsigned>& AtomTable, Object* obj, un
 			BuildAtomTable (AtomTable, pObject, index);
 }
 
-void Document::ExportOB ()
+void Document::ExportOB () const
 {
 	OBMol Mol;
 	map<string, unsigned>::iterator i;
@@ -294,20 +301,21 @@ void Document::ExportOB ()
 	unsigned index = 1;
 	double x, y, z;
 	gchar *old_num_locale;
-	map< string, Object * >::iterator m;
-	stack<map< string, Object * >::iterator> iters;
-	set<Object *> Mols;
-	Object *Cur = this, *Ob;
+	map< string, Object * >::const_iterator m;
+	stack<map< string, Object * >::const_iterator> iters;
+	set<Object const *> Mols;
+	Object const *Cur = this, *Ob;
 	try {
 		ostringstream ofs;
-		GnomeVFSHandle *handle = NULL;
-		GnomeVFSFileSize n;
-		GnomeVFSResult res;
-		res = gnome_vfs_open (&handle, m_filename, GNOME_VFS_OPEN_WRITE);
-		if (res == GNOME_VFS_ERROR_NOT_FOUND)
-			res = gnome_vfs_create (&handle, m_filename, GNOME_VFS_OPEN_WRITE, true, 0666);
-		if (res != GNOME_VFS_OK)
-			throw (int) res;
+		GFile *file = g_file_new_for_uri (m_filename);
+		GError *error = NULL;
+		GOutputStream *output = G_OUTPUT_STREAM (g_file_create (file, G_FILE_CREATE_NONE, NULL, &error));
+		if (error) {
+			g_message ("GIO error: %s", error->message);
+			g_error_free (error);
+			g_object_unref (file);
+			throw (int) 1;
+		}
 		old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 		setlocale (LC_NUMERIC, "C");
 		OBConversion Conv;
@@ -332,7 +340,7 @@ void Document::ExportOB ()
 					Ob = Cur->GetNextChild (m);
 				}
 			}
-			set<Object *>::iterator mi, mend = Mols.end ();
+			set<Object const *>::iterator mi, mend = Mols.end ();
 			unsigned nb = 1;
 			Conv.SetOneObjectOnly (false);
 			for (mi = Mols.begin (); mi != mend; mi++)
@@ -386,19 +394,77 @@ void Document::ExportOB ()
 		}
 		setlocale (LC_NUMERIC, old_num_locale);
 		g_free (old_num_locale);
-		if ((res = gnome_vfs_write (handle, ofs.str ().c_str (), (GnomeVFSFileSize) ofs.str ().size (), &n)) != GNOME_VFS_OK)
-			throw (int) res;
-		gnome_vfs_close (handle);
-		SetReadOnly (false);
+		gsize nb = ofs.str ().size (), n = 0;
+		while (n < nb) {
+			n += g_output_stream_write (output, ofs.str ().c_str () + n, nb - n, NULL, &error);
+			if (error) {
+				g_message ("GIO error: %s", error->message);
+				g_error_free (error);
+				g_object_unref (file);
+				throw (int) 1;
+			}
+		}
+		g_object_unref (file);
+		const_cast <Document *> (this)->SetReadOnly (false);
 	}
 	catch (int n) {
-		fprintf (stderr, "gnome-vfs error #%d\n",n);
+		// TODO: implement a meaningful error reporting system
 	}
 }
 
-void Document::Print (GnomePrintContext *pc, gdouble width, gdouble height)
+void Document::DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const
 {
-	m_pView->Print (pc, width, height);
+	cairo_t *cr;
+	double width, height, x, y, w, h;
+
+	// TODO: support multiple pages
+	cr = gtk_print_context_get_cairo_context (context);
+	width = gtk_print_context_get_width (context);
+	height = gtk_print_context_get_height (context);
+	// adjust position and size
+	GtkWidget* widget = m_pView->GetWidget ();
+	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (widget), "data");
+	ArtDRect rect;
+	pData->GetObjectBounds (this, &rect);
+	double scale = 1.;
+	x = rect.x0;
+	y = rect.y0;
+	w = rect.x1 - rect.x0;
+	h = rect.y1 - rect.y0;
+	cairo_save (cr);
+	cairo_rectangle (cr, 0, 0, width, height);
+	cairo_clip (cr);
+	switch (GetScaleType ()) {
+	case GCU_PRINT_SCALE_NONE:
+		break;
+	case GCU_PRINT_SCALE_FIXED:
+		scale *= Printable::GetScale ();
+		x *= scale;
+		y *= scale;
+		w *= scale;
+		h *= scale;
+		break;
+	case GCU_PRINT_SCALE_AUTO:
+		if (GetHorizFit () && GetVertFit ())
+			scale *= min (width / w, height / h);
+		else if (GetHorizFit ())
+			scale *= width / w;
+		else if (GetVertFit ())
+			scale *= height / h;
+		x *= scale;
+		y *= scale;
+		w *= scale;
+		h *= scale;
+		break;
+	}
+	if (GetHorizCentered ())
+		x -= (width - w) / 2;
+	if (GetVertCentered ())
+		y -= (height - h) / 2;
+	cairo_translate (cr, -x, -y);
+	cairo_scale (cr, scale, scale);
+	m_pView->Render (cr);
+	cairo_restore (cr);
 }
 
 void Document::AddAtom (Atom* pAtom)
@@ -514,18 +580,26 @@ void Document::AddBond (Bond* pBond)
 	}
 }
 
-static int cb_xml_to_vfs (GnomeVFSHandle *handle, const char* buf, int nb)
+static int cb_xml_to_vfs (GOutputStream *output, const char* buf, int nb)
 {
-	GnomeVFSFileSize ndone;
-	return (int) gnome_vfs_write (handle, buf, nb, &ndone);
+	GError *error = NULL;
+	int n = g_output_stream_write (output, buf, nb, NULL, &error);
+	if (error) {
+		g_message ("GIO error: %s", error->message);
+		g_error_free (error);
+	}
+	return n;
 }
 
-void Document::Save ()
+void Document::Save () const
 {
+	if (m_bReadOnly)
+		const_cast <Document *> (this)->SetReadOnly (false);
 	if (!m_filename || !m_bWriteable || m_bReadOnly)
 		return;
 	xmlDocPtr xml = NULL;
 	char *old_num_locale, *old_time_locale;
+	const_cast <Document *> (this)->m_SavedResidues.clear ();
 	
 	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
@@ -538,37 +612,35 @@ void Document::Save ()
 		else {
 			xml = BuildXMLTree ();
 			xmlSetDocCompressMode (xml, CompressionLevel);
-			
-			xmlIndentTreeOutput = true;
-			xmlKeepBlanksDefault (0);
-		
-			GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
-			gnome_vfs_get_file_info (m_filename, info, GNOME_VFS_FILE_INFO_DEFAULT);
 
-			if (GNOME_VFS_FILE_INFO_LOCAL (info)) {
-				gnome_vfs_file_info_unref (info);
-				if (xmlSaveFormatFile (m_filename, xml, true) < 0) /*Error(SAVE)*/;
-			} else {
-				gnome_vfs_file_info_unref (info);
-				xmlOutputBufferPtr buf = xmlAllocOutputBuffer (NULL);
-				GnomeVFSHandle *handle;
-				GnomeVFSResult result = gnome_vfs_open (&handle, m_filename, GNOME_VFS_OPEN_WRITE);
-				if (result == GNOME_VFS_ERROR_NOT_FOUND)
-					result = gnome_vfs_create (&handle, m_filename, GNOME_VFS_OPEN_WRITE, true, 0666);
-				if (result != GNOME_VFS_OK)
-					throw 1;
-				buf->context = handle;
-				buf->closecallback = (xmlOutputCloseCallback) gnome_vfs_close;
-				buf->writecallback = (xmlOutputWriteCallback) cb_xml_to_vfs;
-				int n = xmlSaveFormatFileTo (buf, xml, NULL, true);
-				if (n < 0)
-					throw 1;
-				SetReadOnly (false);
+			if (!CompressionLevel) {
+				xmlIndentTreeOutput = true;
+				xmlKeepBlanksDefault (0);
 			}
+		
+			xmlOutputBufferPtr buf = xmlAllocOutputBuffer (NULL);
+			GFile *file = g_file_new_for_uri (m_filename);
+			GError *error = NULL;
+			GOutputStream *output = G_OUTPUT_STREAM (g_file_create (file, G_FILE_CREATE_NONE, NULL, &error));
+			if (error) {
+				g_message ("GIO error: %s", error->message);
+				g_error_free (error);
+				g_object_unref (file);
+				throw (int) 1;
+			}
+			buf->context = output;
+			buf->closecallback = NULL;
+			buf->writecallback = (xmlOutputWriteCallback) cb_xml_to_vfs;
+			int n = xmlSaveFormatFileTo (buf, xml, NULL, true);
+			g_output_stream_close (output, NULL, NULL);
+			g_object_unref (file);
+			if (n < 0)
+				throw 1;
+			const_cast <Document *> (this)->SetReadOnly (false);
 		}
-		SetDirty (false);
-		m_LastStackSize = m_UndoList.size ();
-		m_OpID = m_UndoList.front ()->GetID ();
+		const_cast <Document *> (this)->SetDirty (false);
+		const_cast <Document *> (this)->m_LastStackSize = m_UndoList.size ();
+		const_cast <Document *> (this)->m_OpID = m_UndoList.front ()->GetID ();
 	}
 	catch (int num) {
 		if (xml)
@@ -580,6 +652,7 @@ void Document::Save ()
 	g_free (old_num_locale);
 	setlocale (LC_TIME, old_time_locale);
 	g_free (old_time_locale);
+	const_cast <Document *> (this)->m_SavedResidues.clear ();
 }
 
 bool Document::Load (xmlNodePtr root)
@@ -686,6 +759,7 @@ bool Document::Load (xmlNodePtr root)
 	}
 	m_pView->Update (this);
 	Update ();
+	m_Empty = !HasChildren ();
 	m_bIsLoading = false;
 	if (m_Window)
 		m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
@@ -698,7 +772,7 @@ void Document::ParseXMLTree (xmlDocPtr xml)
 	Load (xml->children);
 }
 
-xmlDocPtr Document::BuildXMLTree ()
+xmlDocPtr Document::BuildXMLTree () const
 {
 	xmlDocPtr xml;
 	xmlNodePtr node;
@@ -713,8 +787,8 @@ xmlDocPtr Document::BuildXMLTree ()
 	ns = xmlNewNs (xml->children, (xmlChar*) "http://www.nongnu.org/gchempaint", (xmlChar*) "gcp");
 	xmlSetNs (xml->children, ns);
 	if (!g_date_valid (&CreationDate))
-		g_date_set_time_t (&CreationDate, time (NULL));
-	g_date_set_time_t (&RevisionDate, time (NULL));
+		g_date_set_time_t (&const_cast <Document *> (this)->CreationDate, time (NULL));
+	g_date_set_time_t (&const_cast <Document *> (this)->RevisionDate, time (NULL));
 	gchar buf[64];
 	g_date_strftime (buf, sizeof (buf), "%m/%d/%Y", &CreationDate);
 	xmlNewProp (xml->children, (xmlChar*) "creation", (xmlChar*) buf);
@@ -902,7 +976,7 @@ void Document::Remove (Object* pObject)
 				while (object) {
 					m_pView->Remove (object);
 					delete object;
-					object = pObject->GetNextChild (i);
+					object = pObject->GetFirstChild (i);
 				}
 				delete pObject;
 			}
@@ -996,6 +1070,7 @@ void Document::OnUndo ()
 	Update ();
 	EmptyTranslationTable ();
 	SetDirty (m_LastStackSize != m_UndoList.size () || (m_LastStackSize > 0 && m_OpID != m_UndoList.front ()->GetID ()));
+	m_Empty = !HasChildren ();
 }
 
 void Document::OnRedo ()
@@ -1009,7 +1084,7 @@ void Document::OnRedo ()
 		m_RedoList.pop_front ();
 		m_UndoList.push_front (Op);
 		if (m_Window)
-	m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", true);
+			m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", true);
 	}
 	if (m_Window) {
 		if (m_RedoList.empty ())
@@ -1019,6 +1094,7 @@ void Document::OnRedo ()
 	m_bUndoRedo = false;
 	EmptyTranslationTable ();
 	SetDirty (m_LastStackSize != m_UndoList.size () || (m_LastStackSize > 0 && m_OpID != m_UndoList.front ()->GetID ()));
+	m_Empty = !HasChildren ();
 }
 
 void Document::FinishOperation ()
@@ -1032,6 +1108,7 @@ void Document::FinishOperation ()
 	}
 	m_pCurOp = NULL;
 	SetDirty (true);
+	m_Empty = !HasChildren ();
 	if (m_Window) {
 		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", true);
 		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Redo", false);
@@ -1182,7 +1259,7 @@ void Document::ExportImage (string const &filename, const char* type, int resolu
 void Document::SetReadOnly (bool ro)
 {
 	m_bReadOnly = ro;
-	if (!ro && (m_FileType != "application/x-gchempaint")) {
+	if (!ro && (m_FileType != "application/x-gchempaint") && !Loader::GetSaver (m_FileType.c_str ())) {
 		OBFormat *f = OBConversion::FormatFromMIME (m_FileType.c_str ());
 		m_bReadOnly = (f == NULL)? true: f->Flags () & NOTWRITABLE;
 	}
@@ -1209,8 +1286,10 @@ double Document::GetYAlign ()
 
 void Document::SetTheme (Theme *theme)
 {
-	if (!theme)
-		return; // or use the default theme?
+	if (!theme) {
+		m_Theme = NULL;
+		return;
+	}
 	if (m_Theme)
 		m_Theme->RemoveClient (this);
 	m_Theme = theme;
@@ -1317,4 +1396,139 @@ double Document::GetMedianBondLength ()
 	return result;
 }
 
+bool Document::SetProperty (unsigned property, char const *value)
+{
+	switch (property) {
+	case GCU_PROP_DOC_FILENAME:
+		SetFileName (value, m_FileType.c_str ());
+		break;
+	case GCU_PROP_DOC_MIMETYPE:
+		m_FileType = value;
+		break;
+	case GCU_PROP_DOC_TITLE:
+		SetTitle (value);
+		if (m_Window)
+			m_Window->SetTitle (GetTitle ());
+		break;
+	case GCU_PROP_DOC_COMMENT:
+		g_free (m_comment);
+		m_comment = g_strdup (value);
+		break;
+	case GCU_PROP_DOC_CREATOR:
+		g_free (m_author);
+		m_author = g_strdup (value);
+		break;
+	case GCU_PROP_DOC_CREATION_TIME:
+		g_date_set_parse (&CreationDate, value);
+		break;
+	case GCU_PROP_DOC_MODIFICATION_TIME:
+		g_date_set_parse (&RevisionDate, value);
+		break;
+	case GCU_PROP_THEME_BOND_LENGTH: {
+		char *end;
+		double length = strtod (value, &end);
+		if (*end != 0)
+			return false;
+		gcu::Document::m_Scale = m_Theme->GetBondLength () / length; 
+	}
+	}
+	return true;
+}
+
+GtkWindow *Document::GetGtkWindow ()
+{
+	return (m_Window)? m_Window->GetWindow (): NULL;
+}
+
+void Document::SaveResidue (Residue const *r, xmlNodePtr node)
+{
+	if (m_SavedResidues.find (r) == m_SavedResidues.end ()) {
+		m_SavedResidues.insert (r);
+		xmlNewProp (node, (xmlChar const *) "raw", (xmlChar const *) reinterpret_cast <Molecule const*> (r->GetMolecule ())->GetRawFormula ().c_str ());
+		xmlNewProp (node, (xmlChar const *) "generic", (xmlChar const *) (r->GetGeneric ()? "true": "false"));
+		map<string, bool> const &symbols = r->GetSymbols ();
+		map<string, bool>::const_iterator i = symbols.begin (), iend = symbols.end ();
+		string s = (*i).first;
+		for (i++; i != iend; i++) {
+			s += ";";
+			s += (*i).first;
+		}
+		xmlNodePtr child = xmlNewDocNode (node->doc, NULL, (xmlChar const *) "symbols", (xmlChar const *) s.c_str ());
+		xmlAddChild (node, child);
+		map<string, string> const &names = r->GetNames ();
+		map<string, string>::const_iterator j, jend = names.end ();
+		j = names.find ("C");
+		if (j != jend) {
+			child = xmlNewDocNode (node->doc, NULL, (xmlChar const *) "name", (xmlChar const *) (*j).second.c_str ());
+			xmlAddChild (node, child);
+		}
+		for (j = names.begin (); j != jend; j++) {
+			if ((*j).first == "C")
+				continue;
+			child = xmlNewDocNode (node->doc, NULL, (xmlChar const *) "name", (xmlChar const *) (*j).second.c_str ());
+			xmlNodeSetLang (child, (xmlChar const *) (*j).first.c_str ());
+			xmlAddChild (node, child);
+		}
+		child = reinterpret_cast <Molecule const*> (r->GetMolecule ())->Save (node->doc);
+		if (child) {
+			xmlAddChild (node, child);
+		}
+	}
+}
+
+gcu::Residue const *Document::GetResidue (char const *symbol, bool *ambiguous)
+{
+	map<string, SymbolResidue>::iterator i = m_Residues.find (symbol);
+	if (i != m_Residues.end ()) {
+		if (ambiguous)
+			*ambiguous = (*i).second.ambiguous;
+		return (*i).second.res;
+	} else
+		return gcu::Document::GetResidue (symbol, ambiguous);
+}
+
+gcu::Residue *Document::CreateResidue (char const *name, char const *symbol, gcu::Molecule *molecule)
+{
+	Residue *res = NULL;
+	Residue const *r;
+	bool ambiguous;
+	map< string, Object * >::iterator i;
+	Object *obj = molecule->GetFirstChild (i);
+	gcu::Atom *a = NULL;
+	while (obj) {
+		a = dynamic_cast <gcu::Atom *> (obj);
+		if (a && ! a->GetZ ())
+			break;
+		a = NULL;
+		obj = molecule->GetNextChild (i);
+	}
+	if (!a || a->GetBondsNumber () != 1)
+		return NULL;
+	if (strcmp (a->GetId (), "a1")) {
+		Object *o = molecule->GetChild ("a1");
+		if (o) {
+			o->SetId ("at1");
+			a->SetId ("a1");
+			o->SetId ("a1");
+		} else
+			a->SetId ("a1");
+	}
+	double x, y;
+	a->GetCoords (&x, &y);
+	molecule->Move (-x, -y);
+	map <gcu::Atom*, gcu::Bond*>::iterator j;
+	gcu::Bond *b = a->GetFirstBond (j);
+	double angle = b->GetAngle2DRad (a);
+	Matrix2D m (-angle, false);
+	molecule->Transform2D (m, 0., 0.);
+	// does a global residue exists for that symbol?
+	r = static_cast <Residue const*> (Residue::GetResidue (symbol, &ambiguous));
+	if (!r)
+		res = new Residue (name, symbol, dynamic_cast <Molecule *> (molecule), NULL);
+	else {
+		// TODO: add residues specific to the document
+	}
+	return res;
+}
+
 }	//	namespace gcp
